package com.unis.module.coding.service.delivery;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.github.yitter.idgen.YitIdHelper;
import com.unis.framework.common.util.kqbm.KQBMUtils;
import com.unis.module.coding.controller.admin.caxx.vo.CaxxSaveReqVO;
import com.unis.module.coding.controller.admin.cwxx.vo.CwxxSaveReqVO;
import com.unis.module.coding.controller.admin.cwxxyear.vo.CwxxYearSaveReqVO;
import com.unis.module.coding.controller.admin.delivery.vo.CaxxUpdateReqVO;
import com.unis.module.coding.controller.admin.delivery.vo.DeliverySaveVO;
import com.unis.module.coding.controller.admin.dwxx.vo.DwxxSaveReqVO;
import com.unis.module.coding.dal.dataobject.cwxx.CwxxDO;
import com.unis.module.coding.dal.dataobject.cwxxyear.CwxxYearDO;
import com.unis.module.coding.dal.dataobject.dwxx.DwxxDO;
import com.unis.module.coding.dal.dataobject.kqxx.KqxxDO;
import com.unis.module.coding.dal.mysql.cwxx.CwxxMapper;
import com.unis.module.coding.dal.mysql.cwxxyear.CwxxYearMapper;
import com.unis.module.coding.dal.mysql.dwxx.DwxxMapper;
import com.unis.module.coding.dal.mysql.kqxx.KqxxMapper;
import com.unis.module.coding.service.caxx.CaxxService;
import com.unis.module.coding.service.cwxx.CwxxService;
import com.unis.module.coding.service.cwxxyear.CwxxYearService;
import com.unis.module.coding.service.dwxx.DwxxService;
import com.unis.module.coding.service.kqxx.KqxxService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

import static com.unis.framework.common.exception.util.ServiceExceptionUtil.exception;
import static com.unis.module.infra.enums.ErrorCodeConstants.ERROR;

/**
 * @author qingsong.han
 * @description:
 * @create 2025-01-22 15:28
 */
@Service
@Slf4j
public class DeliveryServiceImpl implements DeliveryService {
    @Resource
    private DwxxService dwxxService;
    @Resource
    private KqxxService kqxxService;
    @Resource
    private KqxxMapper kqxxMapper;
    @Resource
    private CaxxService caxxService;
    @Resource
    private CwxxService cwxxService;
    @Resource
    private CwxxMapper cwxxMapper;
    @Resource
    private CwxxYearMapper cwxxYearMapper;
    @Resource
    private CwxxYearService cwxxYearService;
    @Resource
    private DwxxMapper dwxxMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateCaxxByIds(CaxxUpdateReqVO updateReqVO) {
        List<Long> ids = updateReqVO.getIds();
        if (CollectionUtils.isNotEmpty(ids)) {
            ids.forEach(id -> {
                CaxxSaveReqVO updateVO = new CaxxSaveReqVO();
                updateVO.setId(id);
                caxxService.updateCaxx(updateVO);
            });
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Object createByAll(DeliverySaveVO createReqVO) {
        // 单位信息
        if (ObjectUtils.isNotEmpty(createReqVO.getDwxxReqVO())) {
            if (ObjectUtils.isEmpty(createReqVO.getDwxxReqVO().getTyxydm())) {
                throw exception(ERROR, "统一信用编码为空");
            }
            //根据统一信用编码----添加之前查询是否已经添加过该单位
            List<DwxxDO> tyxydm1 = dwxxMapper.selectList(new QueryWrapper<DwxxDO>().eq("TYXYDM", createReqVO.getDwxxReqVO().getTyxydm()));
            Long dwxx = null;
            if (CollectionUtils.isEmpty(tyxydm1)) {
                dwxx = dwxxService.createDwxx(createReqVO.getDwxxReqVO());
            } else {
                log.info("根据统一信用编码查询之前已经添加过该单位({})", tyxydm1.get(0).getId());
                dwxx = tyxydm1.get(0).getId();
            }
            // 库区信息
            if (ObjectUtils.isNotEmpty(createReqVO.getKqxxReqVO())) {
                // set 单位id
                createReqVO.getKqxxReqVO().setDwid(dwxx);
///////////////////////////////////////////////////////////
                /*
                 * 生成库区代码
                 */
                Long count = kqxxMapper.selectCount(new QueryWrapper<KqxxDO>().eq("DWID", dwxx));
                String kqdm = "";
                //同一单位(DELIVERY_DWXX)下多个库区,生成规则 01 02 ...
                if (count == 0 || count == null) {
                    kqdm = "01";
                } else if (count < 10) {
                    kqdm = "0" + ++count;
                } else {
                    kqdm = String.valueOf(count);

                }
                createReqVO.getKqxxReqVO().setKqdm(kqdm);
                //库区编码
                String kqbm = "";
                // 单位信息社会统一信用编码
                String tyxydm = createReqVO.getDwxxReqVO().getTyxydm();
                tyxydm = tyxydm.length() <= 18 ? tyxydm : tyxydm.substring(0, 18);
                //企业性质
                String qyxz = createReqVO.getDwxxReqVO().getQyxz();
                //库区编码
                kqbm = KQBMUtils.createKQBM(qyxz, tyxydm, kqdm);
                createReqVO.getKqxxReqVO().setKqbm(kqbm);
                Long kqxx = kqxxService.createKqxx(createReqVO.getKqxxReqVO());

//////////////////////////////////////////////////////////////////////////////////////
                // 仓房信息
                if (CollectionUtils.isNotEmpty(createReqVO.getCaxxReqVOList())) {
                    //库区编码
                    String finalKqbm = kqbm;
                    createReqVO.getCaxxReqVOList().forEach(caxx -> {
                        caxx.setYxcf(1);
                        caxx.setKqid(kqxx);
                        caxx.setKqmc(createReqVO.getKqxxReqVO().getKqmc());
                        //仓房代码生成
                        String cfdn = KQBMUtils.getCFDM(finalKqbm, caxx.getCfbm());
                        caxx.setCfdm(cfdn);
                        caxxService.createCaxx(caxx);
                    });
                }

                // 财务信息
                CwxxSaveReqVO cwxxReqVO = createReqVO.getCwxxReqVO();
                if (ObjectUtils.isNotEmpty(cwxxReqVO)) {
                    cwxxReqVO.setKqid(kqxx);
                    cwxxReqVO.setDeptId(dwxx);
                    Long cwxx = cwxxService.createCwxx(cwxxReqVO);
                    // 财年信息
                    if (CollectionUtils.isNotEmpty(cwxxReqVO.getCwxxYearList())) {
                        for (CwxxYearSaveReqVO cwxxYear : cwxxReqVO.getCwxxYearList()) {
                            cwxxYear.setCwxxId(cwxx);
                            cwxxYearService.createCwxxYear(cwxxYear);
                        }
                    }
                }
            }
        }
        // 交通运输情况
        return null;
    }
}
